local bump = require "bump/bump"
local anim8 = require "anim8/anim8"
local camera = require "hump/camera"
love.graphics.setDefaultFilter("nearest", "nearest")
local image = love.graphics.newImage("Sprites/8bitSheet.png")
local STI = require("sti")
local level = require("Map/testMap")


local player = {
    x = 50,
    y = 50 ,
    w = 8,
    h = 8,
    img = nil,
    xVel = 0,
    yVel = 0,
    grav = -500,
    acc = 1000,
    maxSpeed = 500,
    factor = 1,
    zoom = 1,
    isGrounded = false
}

Platform = {
    x = 100,
    y = 150,
    w = 300,
    h = 50
}

Platform2 = {
    x = 50,
    y = 500,
    w = 700,
    h = 80

}


function levelLoader(level)
    Map = STI(level)
    local objects = level.layers["solids"].objects
    for i = 1, #objects do
        local obj = objects[i]
        world:add(obj, obj.x, obj.y, obj.width, math.abs(obj.height))
    end
end



function love.load()
    cam = camera()
    world = bump.newWorld()
    Player = player
    levelLoader(level)
    
    world:add(Player, Player.x - Player.w * Player.zoom * 0.5,
             Player.y - Player.h * Player.zoom, Player.w * Player.zoom, Player.h * Player.zoom)
    
    Player.img = image
    animation()
end

function animation()
    local g = anim8.newGrid(8, 8, Player.img:getWidth(), Player.img:getHeight())
    idle = anim8.newAnimation(g("1-13", 1), 0.1)
    run = anim8.newAnimation(g("14-15", 1), 0.1)
    air = anim8.newAnimation(g(24, 1), 0.1)
    fall = anim8.newAnimation(g(25, 1), 0.1)
    currAnim = idle
end

function collide(player, dt)
    local finalX = player.x - player.w * player.zoom * 0.5 + player.xVel * dt
    local finalY = player.y - player.h * player.zoom +player.yVel * dt

    local actualX, actualY, cols, len = world:move(player, finalX, finalY)

    player.isGrounded = false
    for i = 1, len do
        local col = cols[i]

        if col.normal.y == -1 or col.normal.y == 1 then
            player.yVel = 0
        end

        if col.normal.y == -1 then
            player.isGrounded = true
        end
    end

    player.x = actualX + player.w * player.zoom * 0.5
    player.y = actualY + player.h * player.zoom 
end

function appGravity(dt)
    if Player.yVel <= Player.maxSpeed then
        Player.yVel = Player.yVel - Player.grav * dt
    end
end

function playerMove(dt)
    if love.keyboard.isScancodeDown("d") then
        currAnim = run
        if Player.xVel <= Player.maxSpeed then
            Player.xVel = Player.xVel + Player.acc * dt
        end
    elseif love.keyboard.isScancodeDown("a") then
        currAnim = run
        if Player.xVel >= -Player.maxSpeed then
            Player.xVel = Player.xVel - Player.acc * dt
        end
    else
        currAnim = idle
        
        local friction = 1000
        
        if Player.xVel > 0 then
            Player.xVel = Player.xVel - friction * dt
            if Player.xVel < 0 then Player.xVel = 0 end
        end

        if Player.xVel < 0 then
            Player.xVel = Player.xVel + friction * dt
            if Player.xVel > 0 then Player.xVel = 0 end
        end

    end
end

function animChecker()
    if not Player.isGrounded then
        if Player.yVel ~= 0 then
            if Player.yVel < 0 then
                currAnim = air
            elseif Player.yVel > 0 then
                currAnim = fall
            end
        end
    end
end

function love.update(dt)
    currAnim:update(dt)
    playerMove(dt)
    collide(Player, dt)
    appGravity(dt)
    animChecker()
    cam:lookAt(Player.x, Player.y)

    local w = love.graphics.getWidth()
    local h = love.graphics.getHeight()
    local mapW = Map.width * Map.tilewidth
    local mapH = Map.height * Map.tileheight
    -- left border
    if cam.x < w/2 then
        cam.x = w/2
    end
    -- bottom border
    if cam.y < h/2 then
        cam.y = h/2
    end

    

    --right border
    if cam.x > (mapW - w/2) then
        cam.x = (mapW - w/2)
    end
    -- left border
    if cam.y > (mapH - h/2) then
        cam.y = (mapH - h/2)
    end

end

function love.draw()
    cam:attach()
        Map:drawLayer(Map.layers["tiles"])
        currAnim:draw(Player.img, Player.x, Player.y, nil, Player.factor * Player.zoom, Player.zoom, 4, 8)
    cam:detach()
end 

function love.keypressed(key)
    if key == "w" and Player.isGrounded then
        Player.yVel = -300
    end
    if key == "d" then
        Player.factor = 1
    elseif key == "a" then
        Player.factor = -1
    end
end

